﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Client;

namespace TFS_Timesheet
{
    class Program
    {
        static void Main(string[] args)
        {
            Console.WriteLine("TFS Timesheet Report Builder.");

            var request = new RequestData();

            foreach (var arg in args)
            {
                if (arg.StartsWith("/collection:"))
                {
                    request.Collection = new Uri(arg.Substring("/collection:".Length));
                }
                else if (arg.StartsWith("/from:"))
                {
                    request.From = DateTime.Parse(arg.Substring("/from:".Length) + "T00:00:00.0000000");
                }
                else if (arg.StartsWith("/to:"))
                {
                    request.To = DateTime.Parse(arg.Substring("/to:".Length) + "T00:00:00.0000000");
                }
            }

            if (request.Collection == null)
            {
                WriteHelp();
                return;
            }

            var timeEntries = new List<TimeEntry>();

            var teamProjectCollection = new TfsTeamProjectCollection(request.Collection, true);

            var workItemStore = teamProjectCollection.GetService<WorkItemStore>();
            var workItems = workItemStore.Query(string.Format(
                @"SELECT [System.Id], [System.ChangedDate], [System.Title], [System.State] 
                  FROM WorkItems 
                  WHERE [System.ChangedDate] >= '{0}' AND [System.ChangedDate] < '{1}'
                  ORDER BY [System.ChangedDate]", request.From.ToLongDateString(), request.To.AddDays(1).ToLongDateString()));
            foreach (WorkItem workItem in workItems)
            {
                if (workItem == null || !workItem.IsValid()) continue;

                foreach (Revision rev in workItem.Revisions)
                {
                    foreach (Field field in rev.Fields)
                    {
                        if (field.Name == "Completed Work" && 
                            field.OriginalValue != field.Value && 
                            Convert.ToDateTime(rev.Fields[CoreField.ChangedDate].Value) >= request.From &&
                            Convert.ToDateTime(rev.Fields[CoreField.ChangedDate].Value) < request.To.AddDays(1))
                        {
                            var entry = new TimeEntry();
                            entry.Id = workItem.Id;
                            entry.Date = Convert.ToDateTime(rev.Fields[CoreField.ChangedDate].Value);
                            entry.Name = rev.Fields[CoreField.ChangedBy].Value.ToString();
                            entry.Title = workItem.Title;
                            entry.Hours = Convert.ToDouble(field.Value) - Convert.ToDouble(field.OriginalValue);
                            if (entry.Hours > 0)
                            {
                                timeEntries.Add(entry);
                            }
                        }
                    }
                }
            }

            WriteTimesheet(timeEntries);
        }

        private static void WriteHelp()
        {
            Console.WriteLine("TFS_Timesheet.exe /collection:<TFSURL> /from:<StartDate> /to:<EndDate>");
            Console.WriteLine("TFS_Timesheet.exe /collection:http://server:8080/tfs/defaultcollection /from:2012-03-01 /to:2012-03-31");
        }

        private static void WriteTimesheet(List<TimeEntry> timeEntries)
        {
            var workItems = timeEntries.Select(t => t.Id).Distinct().OrderBy(t => t).ToList();
            var dates = timeEntries.Select(t => t.Date.Date).Distinct().OrderBy(t => t).ToList();

            using(var timesheet = File.CreateText("timesheet_report.html"))
            {
                timesheet.Write("<html><head><title>TFS Timesheet Report</title><style>\n"+Properties.Resources.Css+"\n</style>");
                timesheet.Write("<meta content=\"text/html; charset=utf-8\" http-equiv=\"content-type\" />");
                timesheet.Write("<head><body><table><thead><tr><th class='id'>ID</th><th class='title'>Title</th><th>&nbsp;</th>");

                foreach (var date in dates)
                {
                    timesheet.Write("<th>" + date.ToShortDateString() + "</th>");
                }
                timesheet.WriteLine("</tr></thead><tbody>");

                // Общее время.
                timesheet.Write("<tr><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td>");
                foreach (var date in dates)
                {
                    var entries = timeEntries.Where(t => t.Date.Date == date).ToList();
                    var hours = entries.Aggregate(0.0, (t, i) => t + i.Hours);
                    timesheet.Write("<td class='total'>" + hours + "</td>");
                }
                timesheet.Write("</tr>");

                // Распределение времени по задачам.
                foreach (var wid in workItems)
                {
                    timesheet.Write("<tr>");
                    timesheet.Write("<td class='id'>" + wid + "</td>");
                    var title = timeEntries.First(t => t.Id == wid).Title;
                    timesheet.Write("<td class='title'>" + title + "</td>");

                    var entries = timeEntries.Where(t => t.Id == wid).ToList();
                    var hours = entries.Aggregate(0.0, (t, i) => t + i.Hours);
                    timesheet.Write("<td class='total'>" + hours + "</td>");

                    foreach (var date in dates)
                    {
                        entries = timeEntries.Where(t => t.Id == wid && t.Date.Date == date).ToList();
                        hours = entries.Aggregate(0.0, (t, i) => t + i.Hours);

                        timesheet.Write("<td>");
                        if (hours > 0)
                        {
                            timesheet.Write(hours);
                        }
                        else
                        {
                            timesheet.Write("&nbsp;");
                        }
                        timesheet.Write("</td>");
                    }

                    timesheet.WriteLine("</tr>");
                }
                timesheet.WriteLine("</tbody></table></body></html>");
            }
        }
    }
}
